@@ -38,9 +38,9 @@ for i in xrange(64):
 
     u = uimod.ui.load()
     if cmd_quiet or cmd_debug or cmd_verbose:
-        u.setconfig('ui', 'quiet', str(bool(cmd_quiet)))
-        u.setconfig('ui', 'verbose', str(bool(cmd_verbose)))
-        u.setconfig('ui', 'debug', str(bool(cmd_debug)))
+        u.setconfig(b'ui', b'quiet', pycompat.bytestr(bool(cmd_quiet)))
+        u.setconfig(b'ui', b'verbose', pycompat.bytestr(bool(cmd_verbose)))
+        u.setconfig(b'ui', b'debug', pycompat.bytestr(bool(cmd_debug)))
 
     check = ''
     if u.debugflag:
